"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getDefaults;

var _utils = require("./utils");

var _validate = require("./validate");

const {
  isArray
} = Array;
const {
  assign,
  entries
} = Object; // Recursively walks a schema and collects defaults. When no schema is provided,
// the default config schema is used.

function getDefaultsFromSchema(schema) {
  if (!schema || typeof schema.$ref === 'string') {
    var _schema$$ref;

    // get the schema from ajv
    return getDefaultsFromSchema((0, _validate.getSchema)((_schema$$ref = schema === null || schema === void 0 ? void 0 : schema.$ref) !== null && _schema$$ref !== void 0 ? _schema$$ref : '/config'));
  } else if (schema.default != null) {
    // return the default for this schema
    return schema.default;
  } else if (schema.type === 'object' && schema.properties) {
    // return an object of default properties
    return entries(schema.properties).reduce((acc, [prop, schema]) => {
      let def = getDefaultsFromSchema(schema);
      return def != null ? assign(acc || {}, {
        [prop]: def
      }) : acc;
    }, undefined);
  } else {
    return undefined;
  }
}

function getDefaults(overrides = {}) {
  return (0, _utils.merge)([getDefaultsFromSchema(), overrides], (path, prev, next) => {
    // override default array instead of merging
    return isArray(next) && [path, next];
  });
}