"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.colors = exports.ANSI_COLORS = exports.ANSI_REG = void 0;
const {
  assign,
  entries
} = Object; // matches ansi escape sequences

const ANSI_REG = new RegExp('[\\u001B\\u009B][[\\]()#;?]*((?:(?:[a-zA-Z\\d]*(?:;[-a-zA-Z\\d\\/#&.:=?%@~_]*)*)?\\u0007)' + '|(?:(?:\\d{1,4}(?:;\\d{0,4})*)?[\\dA-PR-TZcf-ntqry=><~]))', 'g'); // color names by ansi escape code

exports.ANSI_REG = ANSI_REG;
const ANSI_COLORS = {
  '31m': 'red',
  '33m': 'yellow',
  '34m': 'blue',
  '35m': 'magenta',
  '90m': 'grey'
}; // colorize each line of a string using an ansi escape sequence

exports.ANSI_COLORS = ANSI_COLORS;
const LINE_REG = /^.*$/gm;

function colorize(code, str) {
  return str.replace(LINE_REG, line => `\u001b[${code}${line}\u001b[39m`);
} // map ansi colors to bound colorize functions


const colors = entries(ANSI_COLORS).reduce((colors, [code, name]) => {
  return assign(colors, {
    [name]: colorize.bind(null, code)
  });
}, {});
exports.colors = colors;